#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk_imlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <linux/videodev.h>
#include "bttv.h"
#include "fake_capture.h"

// video chapture quality
static struct video_channel vch;
static struct video_mbuf vbuf;
static struct video_mmap vmm;

// point for chapture data
static guchar* buf;

guchar b_buf[ V4L_WIDTH * V4L_HEIGHT *3];

// filediscript of video chapture device
static gint fd;

// BGR -> RGB utility for change function
inline void swap_rb(gint i, gint j, guchar* buf)
{
  guchar tmp;

  tmp = buf[i+2];
  buf[i+2] = buf[i];
  buf[i] = tmp;
}

// initialize device ,and function for chapture start
void bttv_driver_init()
{
  // Video device open
  if ((fd = open(V4L_DEVICE, O_RDWR)) <= 0)
    {
      g_error("Video Device Open");
      exit(-1);
    }

  // mmap information get
  if (ioctl(fd, VIDIOCGMBUF, &vbuf) < 0)
    {
      g_error("VIDIOCGMBUF");
      exit(-1);

      // double buffer and suppose
      if (vbuf.frames < 2)
	{
	  g_error("NO DOUBLE BUFFER");
	  exit(-1);
	}
    }

  // mmap
  if ((buf = (unsigned char*)mmap(0, vbuf.size, PROT_READ | PROT_WRITE,
				  MAP_SHARED, fd, 0)) == MAP_FAILED)
    {
      g_error("MMAP");
      exit(-1);
    }

  // setting input signal
  vch.channel = 0;
  vch.norm = VIDEO_MODE_NTSC;
  if (ioctl(fd, VIDIOCSCHAN, &vch) < 0)
    {
      g_error("VIDIOCSCHAN");
      exit(-1);
    }

  // detail setting for chapture data
  vmm.width = V4L_WIDTH;
  vmm.height = V4L_HEIGHT;
  vmm.format = VIDEO_PALETTE_RGB24;

  // start chapture buffer 0
  vmm.frame = 0;
  if (ioctl(fd, VIDIOCMCAPTURE, &vmm) < 0)
    {
      g_error("VIDIOCMCAPTURE(0)");
      exit(-1);
    }

  // start chapture buffer 1
  vmm.frame = 1;
  if (ioctl(fd, VIDIOCMCAPTURE, &vmm) < 0)
    {
      g_error("VIDIOCMCAPTURE(1)");
      exit(-1);
    }

  // first chapture is buffer 0
  vmm.frame = 0;
}

/* chapture next frame
   set pull number if there is now image
   how to use example : current = get_next_frame(current);
*/
GdkImlibImage*
get_next_frame(GdkImlibImage* current)
{
  gint i;
  //gint x=0,y=0;
  guchar* b;

  // wait for chapture end of 1 frame
  if (ioctl(fd, VIDIOCSYNC, &vmm.frame) < 0)
    {
      g_error("VIDIOCSYNC");
      exit(-1);
    }

  b = buf + vbuf.offsets[vmm.frame];

  //全画像保存
  for (i = 0; i < V4L_WIDTH * V4L_HEIGHT * 3; i++)
    b[i] = b_buf[i];

  //画像を映像として読み込み
  fake_capture();

  // BGR -> RGB data change
  for (i = 0; i < V4L_WIDTH * V4L_HEIGHT * 3; i += 3) {
    swap_rb(i, i+2, b);  
  }

  //blue (x,y) = (160,120)
  /*
  for(y=-2; y<3; y++){
    for(x=-2; x<3; x++){
      b[320*3*(119+y)+477+x*3+1]=255;   //blue
      b[320*3*(119+y)+477+x*3+2]=255; //red
      b[320*3*(119+y)+477+x*3+3]=0; //green
    }
  }
  */
  
  // chapture continue of next frame
  if (ioctl(fd, VIDIOCMCAPTURE, &vmm) < 0)
    {
      g_error("VIDIOCMCAPTURE");
      exit(-1);
    }

  // change buffer
  vmm.frame = (vmm.frame == 0) ? 1 : 0;

  // release held image ,and return new image
  if (current) gdk_imlib_kill_image(current);
  return gdk_imlib_create_image_from_data(b, NULL, V4L_WIDTH, V4L_HEIGHT);
}
